# Als Dynamic Router for Express

**Being tested.**

- [Als Dynamic Router for Express](#als-dynamic-router-for-express)
  - [About](#about)
    - [new in 0.4](#new-in-04)
    - [new in 0.3.5](#new-in-035)
    - [new in 0.3.1](#new-in-031)
    - [new in 0.3](#new-in-03)
    - [new in 0.2](#new-in-02)
  - [Define the router](#define-the-router)
  - [The routes](#the-routes)
  - [Middleware](#middleware)
  - [Dynamic middleware](#dynamic-middleware)
  - [Router](#router)
  - [Static routes](#static-routes)
  - [isError function](#iserror-function)

## About
Dynamic router works as express middleware and offers route system based on files tree.

### new in 0.4
   * static,mw,router,isError are separeted files exported with main file
   * bugs in router are fixed

### new in 0.3.5
  * static - staticFn can be array
  * fixed routes with dynamic parameters and static which comes now before dynamic


### new in 0.3.1
  * router bug fixed - now it's array of objects ``[{url,path,method},...]``

### new in 0.3
  * Static path - relative to project root
  * Added content type for sendind static files
  * bug with mw on higher level and not private fixed


### new in 0.2
* fixed bug with mw and private

What Dynamic Router can?
1. Manage routes in Express by directory tree
2. Add dynamic routers (you can use polymorphism)
3. Add dynamic middleware
4. Set access modifiers to each route or group of routes
5. Add dynamic static folders
6. Add custom 404 page


## Define the router

Define the router:
```javascript
let {join} = require('path')
let {error,mw,Router,static} = require('als-dynamic-router')
let express = require('express')
let app = express()

// Define folder for routes
new Router(app,join(__dirname,'routes'))

app.listen(3000)
```


## The routes
Now inside your routes folder you can use files which folowing this name convension:``route.method.js``.
The methods allowed: get|post|put|patch|delete.
For example ``get.js`` or ``some.post.js``.
Here some examples:

| File location                 | Route               |
|-------------------------------|---------------------|
| \routes\get.js                | GET /               |
| \routes\users\get.js          | GET /users          |
| \routes\users\#$user.get.js   | GET /users/:user    |
| \routes\users\$user.delete.js | DELETE /users/:user |
| \routes\users\$user.post.js   | POST /users/:user   |
| \routes\#login.get.js         | GET /login          |

The ``$`` in filename means parameter ``:``. 
The ``#`` in filname means ``req.private = true``.

Here example for route function:
```javascript
module.exports = function(req,res) {
   return res.end('<div>Hello world</div>')
}
```

## Middleware
You can use middleware in each route and middleware for multiple routes. 

Here example for using middleware in route:
```javascript
let auth = require('./auth')
module.exports = [auth,function(req,res) {
   return res.end('<div>Hello world</div>')
}]
```

In case of middleware for multiple routes, you need to use ``mw.js`` or ``#mw.js``. 
1. ``mw.js`` - apply middleware to all routes inside it's folder and subfolders
2. ``#mw.js`` - apply middleware only to all routes inside it's folder

Example for mw:
```javascript
let auth = require('./auth')
let layout = require('../../layouts/some')
module.exports = function(req,res,next) {
   req.layout = layout
   if(req.private && req.auth) return next()
   return res.redirect('/login')
}
```

## Dynamic middleware

Dynamic middleware let you to use dynamic middleware that depend on some conditions.
Is an static method which immitate Express middleware inside function in route. 
Here is the example:

```javascript
let {mw} = require('als-dynamic-router')
let main = (req,res,next) => {
   req.layout = require('../layouts/basic/layout')
   return next()
}
let test = (req,res,next) => {
   console.log('Hello from private')
   return next()
}

module.exports = function(req,res,next) {
   let privateRoute = mw([test,main])
   let justRoute = mw([main])
   if(req.private) return privateRoute(req,res,next)
   else return justRoute(req,res,next)
}
```


## Router
You can use router inside route as ``router.js`` or ``#router.js``. 
In case of ``#router.js``, all routes will be private.

The router.js has to return array with routes in format: ``[{url,method,path},...]``.

Here example:
/pages
```javascript
let {join} = require('./path')
let get = require('../../crud/get')
let test = require('../../crud/test')
module.exports = [
   {url:'/',method:'post',path:join(__dirname,'..','..','crud','post.js')},
   {url:'/',method:'get',path:get},
   {url:'/test',method:'get',[(req,res,next)=>{next()},path:get]},
]
```
As shown above, path can be path to function,function itself or array of functions.


## Static routes

Syntax:
```javascript
let {static} = require('als-dynamic-router')

// Define mw for static directories
// This route must be the last one after other routes
app.get('*',[(req,res,next)=> {
   req.statics = {'/':join(__dirname,'public')}
},static])
```

static function check if req.url inlcuded inside req.statics. If true, return file with suitable mime type. If not, return ``res.status(404).end('not found')``.

The good news, are that now you can use statics dynamicly. 



## isError function

try catch for routes. 

```javascript
let {isError} = require('als-dynamic-router')
isError(fn,errorFn)
```

if errorFn not given, ``return res.status(500).end('Internal server error')``

Example for route:

```javascript
let {isError} = require('als-dynamic-router')
module.exports = function(req,res,next) {
   return isError(
      (req,res) => { // code for try  },
      (req,res) => { // code for error },
      )
}
```
Example for mw:

```javascript
let {isError} = require('als-dynamic-router')
module.exports = [
   function(req,res,next) {
   return isError(
      (req,res,next) => { // code for try  },
      (req,res) => { // code for error },
      )
   },
   function(req,res) {}
]
```